const FS = require('fs');
const Util = require('util');
const mkdirp = require('make-dir');
const Path = require('path');

const FS2 = Object.entries(FS).reduce((acc, [k, v]) => {
    if(typeof v === 'function' && /^[a-z]/.test(k) && !k.endsWith('Sync')) {
        acc[k] = Util.promisify(v);
    } else {
        acc[k] = v;
    }
    return acc;
}, Object.create(null));

FS2.hasAccess = (path, mode) => FS2.access(path, mode)
    .then(
        () => true,
        err => {
            // if file does not exist or permission is denied, return false, otherwise throw
            if(err.code === 'ENOENT' || err.code === 'EACCES') {
                return false;
            }
            throw err;
        }
    );

FS2.exists = path => FS2.hasAccess(path, FS2.constants.F_OK);

FS2.readText = path => FS2.readFile(path, {encoding: 'utf8'});

FS2.writeText = (path,contents) => {
    const write = () => FS2.writeFile(path, contents, {encoding: 'utf8'});
    
    return write().catch(async err => {
        if(err.code === 'ENOENT') {
            await mkdirp(Path.dirname(path));
            return write();
        }
        throw err;
    });
};

FS2.closest = async (filename, dir=process.cwd()) => {
    let path = Path.join(dir, filename);
    if(await FS2.exists(path)) {
        return path;
    }
    if(dir === '/') return null;
    return FS2.closest(filename, Path.dirname(dir));
};

module.exports = FS2;