'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.change = change;
exports.focus = focus;
exports.blur = blur;
exports.mouseEnter = mouseEnter;
exports.mouseLeave = mouseLeave;
exports.submit = submit;
exports.asyncValidation = asyncValidation;
exports.saveState = saveState;

var _actionTypes = require('./actionTypes');

var _actionTypes2 = _interopRequireDefault(_actionTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Creates a flux standard action.
 *
 * @param {string} type Identifies to the consumer the nature of the action that has occurred.
 * @param {*=} payload The payload of the action.
 * @param {*=} meta Extra information that is not part of the payload.
 * @returns {Object} Action object.
 * @see https://github.com/acdlite/flux-standard-action
 */
function fluxStandardAction(type, payload, meta) {
    let action = { type };
    action.payload = payload;
    if (payload instanceof Error) {
        action.error = true;
    }
    if (meta !== undefined) {
        action.meta = meta;
    }
    return action;
}

function change(formId, name, value) {
    return fluxStandardAction(_actionTypes2.default.CHANGE, { formId, name, value });
}

function focus(formId, name) {
    return fluxStandardAction(_actionTypes2.default.FOCUS, { formId, name, isFocused: true });
}

function blur(formId, name) {
    return fluxStandardAction(_actionTypes2.default.FOCUS, { formId, name, isFocused: false });
}

function mouseEnter(formId, name) {
    return fluxStandardAction(_actionTypes2.default.HOVER, { formId, name, isHovering: true });
}

function mouseLeave(formId, name) {
    return fluxStandardAction(_actionTypes2.default.HOVER, { formId, name, isHovering: false });
}

function submit(formId) {
    return fluxStandardAction(_actionTypes2.default.SUBMIT, { formId });
}

function asyncValidation(formId, name, complete) {
    return fluxStandardAction(_actionTypes2.default.ASYNC_VALIDATION, { formId, name, complete });
}

function saveState(formId, data) {
    return fluxStandardAction(_actionTypes2.default.SAVE_STATE, { formId, data });
}