'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.tap = exports.thru = exports.groupBy = exports.flatMap = exports.filterAsync = exports.map = exports.flatten = undefined;
exports.arrayWithout = arrayWithout;
exports.arraySplice = arraySplice;
exports.arrayIncludes = arrayIncludes;

var _isType = require('./isType');

var _bindable = require('./bindable');

var _bindable2 = _interopRequireDefault(_bindable);

var _value = require('./value');

var _collection = require('./collection');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const flatten = exports.flatten = (0, _bindable2.default)(arrayOfArrays => Array.prototype.concat(...arrayOfArrays));

const map = exports.map = (0, _bindable2.default)((array, callback) => (0, _collection.toArray)(array).map(callback));

const filterAsync = exports.filterAsync = (0, _bindable2.default)((array, mapCb, filterCb = _value.identity) => {
    return Promise.all(array.map(mapCb)).then(results => array.filter((_, i) => filterCb(results[i])));
});

/**
 * Map and flatten
 */
const flatMap = exports.flatMap = (0, _bindable2.default)((array, callback) => {
    var _context;

    return (_context = map.call(array, callback), flatten).call(_context);
});

const groupBy = exports.groupBy = (0, _bindable2.default)(function groupBy(array, grouper) {
    return array.reduce((acc, val, idx) => {
        let key = grouper(val, idx);
        if (acc[key]) {
            acc[key].push(val);
        } else {
            acc[key] = [val];
        }
        return acc;
    }, Object.create(null));
});

const thru = exports.thru = (0, _bindable2.default)((array, callback) => callback(array));

const tap = exports.tap = (0, _bindable2.default)((array, callback) => {
    callback(array);
    return array;
});

/**
 * Removes one instance of `value` from `array`, without mutating the original array. Uses loose comparison.
 *
 * @param {Array} array Array to remove value from
 * @param {*|Function} value Value to remove
 * @returns {Array} Array with `value` removed
 */
function arrayWithout(array, value) {
    let func = value;
    if (typeof value !== 'function') {
        func = (v, i) => v == value;
    }
    for (let i = 0; i < array.length; ++i) {
        if (func(array[i], i)) {
            return arraySplice(array, i);
        }
    }
    return array;
}

/**
 * Removes an index from an array without mutating the original array.
 *
 * @param {Array} array Array to remove value from
 * @param {Number} index Index to remove
 * @param {Number} count
 * @param {Array} replaceWith
 * @returns {Array} Array with `value` removed
 */
function arraySplice(array, index, count = 1, replaceWith = []) {
    if (index < array.length) {
        let copy = [...array];
        copy.splice(index, count, ...replaceWith);
        return copy;
    }
    return array;
}

/**
 * Checks if `value` is in collection. Uses loose comparison.
 *
 * @param {Array} array
 * @param {*} value
 * @returns {boolean}
 */
function arrayIncludes(array, value) {
    for (let val of array) {
        if (val == value) {
            return true;
        }
    }
    return false;
}