'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = connectField;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactRedux = require('react-redux');

var _util = require('./util');

var util = _interopRequireWildcard(_util);

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _getOr = require('lodash/fp/getOr');

var _getOr2 = _interopRequireDefault(_getOr);

var _recompose = require('recompose');

var _namespace = require('./namespace');

var _namespace2 = _interopRequireDefault(_namespace);

var _reselect = require('reselect');

var _shallowEqual = require('./shallowEqual');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _actionCreators = require('./actionCreators');

var actions = _interopRequireWildcard(_actionCreators);

var _DeepMap = require('./DeepMap');

var _DeepMap2 = _interopRequireDefault(_DeepMap);

var _consts = require('./consts');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { toPath } = _;


const defaultValueGetter = (_, p) => p.defaultValue;
const stateGetter = (s, p) => _.get(s, [_namespace2.default, p.form.id], _consts.emptyObject);

const ruleCache = new _DeepMap2.default();
// const notFound = Symbol('NotFound');
// const noError = Symbol('NoError');
// const pending = Symbol('Pending');

const errTypeToProp = {
    // 'err': 'errors',
    'error': 'errors',
    // 'warn': 'warnings',
    'warning': 'warnings'
};

// window.RULE_CACHE = ruleCache;

function getErrorMessages(result, rule, args) {
    if (result === true) {
        return _consts.emptyArray;
    }
    if (result === false) {
        if (_.isFunction(rule.message)) {
            return util.array(rule.message(...args));
        } else {
            return util.array(rule.message);
        }
    }
    if (result) {
        // if result is not a boolean, it ought to be an error message or blank
        return util.array(result);
    }
    return _consts.emptyArray;
}

function getErrors(value, rules, formData, dispatch, formId, name, pendingValidations) {
    const getValue = f => _.get(formData, f);

    // console.log(formId,name,pendingValidations);

    let props = {
        errors: [],
        warnings: []
    };

    if (rules.length > 1) {
        // sort async functions to the end so that they can be skipped if other validation rules are already failing
        rules = [...rules].sort((a, b) => {
            if (a.isAsync === b.isAsync) return 0;
            return a.isAsync ? 1 : -1;
        });
    }

    for (let rule of rules) {
        if (rule.isOptional && util.isEmpty(value)) {
            continue;
        }

        let dependsOn = util.array(rule.dependsOn);
        let args = [value];
        if (dependsOn.length) {
            args.push(...dependsOn.map(getValue));
        }

        if (!rule.precondition(...args)) {
            continue;
        }

        let errKey = errTypeToProp[rule.type];

        if (!errKey) {
            throw new Error(`Unsupported rule type: ${rule.type}`);
        }
        let cacheKey = [rule, formId, name];

        if (rule.isAsync) {
            cacheKey.push(...args);
        }

        let result = ruleCache.get(cacheKey);

        if (result) {
            if (rule.isAsync) {
                // fixme: rule.compare isn't used here because we want to cache more than one entry
                props[errKey].push(...result);
                continue;
            } else {
                let [lastArgs, lastMessages] = result;

                if (rule.compare(args, lastArgs)) {
                    props[errKey].push(...lastMessages);
                    continue;
                }
            }
        }

        if (rule.isAsync) {
            if (rule.type === 'error' && props[errKey].length) {
                // skip if other error rules are already failing
                continue;
            }

            ruleCache.set(cacheKey, _consts.emptyArray);
            dispatch(actions.asyncValidation(formId, name, false));
            rule.validate(...args).then(result => {
                ruleCache.set(cacheKey, getErrorMessages(result, rule, args));
                dispatch(actions.asyncValidation(formId, name, true));
            }, () => {
                ruleCache.delete(cacheKey);
                dispatch(actions.asyncValidation(formId, name, true));
            });
        } else {
            let result = rule.validate(...args);
            let messages = getErrorMessages(result, rule, args);
            props[errKey].push(...getErrorMessages(result, rule, args));
            ruleCache.set(cacheKey, [args, messages]);
        }
    }

    return props;
}

function mapDispatchToProps(dispatch, formId, name) {
    return {
        actions: {
            change(value) {
                dispatch(actions.change(formId, name, value));
            }
        },
        events: {
            onChange(ev) {
                dispatch(actions.change(formId, name, ev.target.value));
            },
            onCheck(ev) {
                dispatch(actions.change(formId, name, ev.target.checked));
            },
            // TODO: add onCheckList for checkbox arrays
            onFocus() {
                dispatch(actions.focus(formId, name));
            },
            onBlur() {
                dispatch(actions.blur(formId, name));
            },
            onMouseEnter() {
                dispatch(actions.mouseEnter(formId, name));
            },
            onMouseLeave() {
                dispatch(actions.mouseLeave(formId, name));
            },
            onSubmit() {
                dispatch(actions.submit(formId));
            }
        }
    };
}

function connectField() {
    return (0, _recompose.compose)(_recompose.toClass, (0, _recompose.getContext)({ form: _propTypes2.default.object }), (0, _recompose.withPropsOnChange)(['name', 'rules', 'formId'], props => {
        // console.log(props.name,'changing');
        const form = props.form || {
            id: props.formId,
            rules: [],
            fields: null,
            inputs: null
        };
        const fieldRules = util.array(props.rules);
        const baseRules = form.rules ? util.array(util.glob(form.rules, props.name, [])) : [];
        return {
            form,
            rules: _.concat(baseRules, fieldRules)
        };
    }),
    // connect(require('./mapStateToProps'), require('./mapDispatchToProps')),
    (0, _reactRedux.connectAdvanced)(selectorFactory), (0, _recompose.withPropsOnChange)(['name'], ({ name, form }) => {
        let props = {};
        if (form.fields) {
            props.ref = node => {
                if (node) {
                    form.fields.set(name, node);
                } else {
                    form.fields.delete(name);
                }
            };
        }
        if (form.inputs) {
            props.focusRef = node => {
                if (node) {
                    form.inputs.set(name, node);
                } else {
                    form.inputs.delete(name);
                }
            };
        }
        return props;
    }));
}

function selectorFactory(dispatch, factoryOptions) {
    let dispatchSelector = (0, _reselect.defaultMemoize)(mapDispatchToProps);
    let prevProps = _consts.emptyObject;

    const namePathSelector = (0, _reselect.createSelector)((_, p) => p.name, toPath);
    const stateUiSelector = util.createDeepEqualSelector([stateGetter, namePathSelector], (state, np) => Object.assign({
        isFocused: false,
        isHovering: false,
        mouseEntered: false,
        mouseLeft: false,
        wasFocused: false,
        wasBlurred: false,
        wasChanged: false,
        pendingValidations: 0
    }, _.get(state, ['ui', ...np], _consts.emptyObject)));
    const dataGetter = (0, _reselect.createSelector)(stateGetter, (0, _getOr2.default)(_consts.emptyObject, 'data'));
    const valueSelector = (0, _reselect.createSelector)([dataGetter, namePathSelector, defaultValueGetter], (data, np, dv) => _.get(data, np, dv));
    const initialSelector = (0, _reselect.createSelector)(stateGetter, (0, _getOr2.default)(_consts.emptyObject, 'initial'));
    const initialValueSelector = (0, _reselect.createSelector)([initialSelector, namePathSelector, defaultValueGetter], (init, np, dv) => _.get(init, np, dv));
    const pendingValidationsSelector = (0, _reselect.createSelector)(stateUiSelector, ui => ui.pendingValidations);

    const errorSelector = util.createDeepEqualSelector([valueSelector, (_, p) => p.rules, dataGetter, () => dispatch, (_, p) => p.form.id, (_, p) => p.name, pendingValidationsSelector], getErrors);

    const isDirtySelector = (0, _reselect.createSelector)([valueSelector, initialValueSelector], (value, initialValue) => !_.isEqual(value, initialValue));
    // const isValidSelector = createSelector(errorSelector, errors => errors.length === 0);
    const isEmptySelector = (0, _reselect.createSelector)([valueSelector, defaultValueGetter], _.isEqual);
    const formValidatedSelector = (0, _reselect.createSelector)(stateGetter, state => _.get(state, 'submit', 0) > 0);

    const uiSelector = util.createDeepEqualSelector([stateUiSelector, isDirtySelector, isEmptySelector, formValidatedSelector], (ui, isDirty, isEmpty, formValidated) => ({
        isFocused: ui.isFocused,
        isHovering: ui.isHovering,
        mouseEntered: ui.mouseEntered,
        mouseLeft: ui.mouseLeft,
        wasFocused: ui.wasFocused,
        wasBlurred: ui.wasBlurred,
        wasChanged: ui.wasChanged,
        isValidating: ui.pendingValidations > 0,
        isDirty,
        isEmpty,
        formValidated
    }));

    return (state, props) => {
        let nextProps = _extends({
            value: valueSelector(state, props),
            ui: uiSelector(state, props)
        }, errorSelector(state, props), dispatchSelector(dispatch, props.form.id, props.name), props);

        if ((0, _shallowEqual2.default)(prevProps, nextProps)) {
            return prevProps;
        }

        return prevProps = nextProps;
    };
}