'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = connectForm;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactRedux = require('react-redux');

var _util = require('./util');

var util = _interopRequireWildcard(_util);

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _recompose = require('recompose');

var _namespace = require('./namespace');

var _namespace2 = _interopRequireDefault(_namespace);

var _actionCreators = require('./actionCreators');

var actions = _interopRequireWildcard(_actionCreators);

var _shortid = require('shortid');

var _shortid2 = _interopRequireDefault(_shortid);

var _reselect = require('reselect');

var _shallowEqual = require('./shallowEqual');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _consts = require('./consts');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const stateGetter = (s, p) => _.get(s, [_namespace2.default, p.form.id], _consts.emptyObject);

function focusInput(input) {
    input.focus(); // TODO: check if input is an element, otherwise throw warning if it's a React component
    if (input.scrollIntoViewIfNeeded) {
        input.scrollIntoViewIfNeeded(false);
    }
}

const contextTypes = {
    form: _propTypes2.default.object
};

function connectForm(options) {
    return (0, _recompose.compose)((0, _recompose.withContext)(contextTypes, props => {
        let rules = props.rules || options.rules;
        return {
            form: {
                id: props.id || options.id || _shortid2.default.generate(),
                rules: rules ? util.unflatten(rules) : _consts.emptyObject,
                fields: new Map(),
                inputs: new Map()
            }
        };
    }), (0, _recompose.getContext)(contextTypes),
    // connect(mapStateToProps, mapDispatchToProps),
    (0, _reactRedux.connectAdvanced)(selectorFactory));
}

function selectorFactory(dispatch, factoryOptions) {
    const dataSelector = (0, _reselect.createSelector)(stateGetter, state => _.get(state, 'data', _consts.emptyObject));
    const initialSelector = (0, _reselect.createSelector)(stateGetter, state => _.get(state, 'initial', _consts.emptyObject));
    const dirtySelector = (0, _reselect.defaultMemoize)((d, i) => !_.isEqual(d, i));
    const dirtyDirtySelector = (0, _reselect.createSelector)(dataSelector, initialSelector, (d, i) => () => dirtySelector(d, i));
    let prevProps = {};

    //Memoized Methods

    const validate = (0, _reselect.defaultMemoize)((dispatch, form) => () => {
        dispatch(actions.submit(form.id));
        return Array.from(form.fields.values()).every(f => f.props.errors.length === 0);
    });

    const setField = (0, _reselect.defaultMemoize)((dispatch, form) => (name, value, data) => {
        if (_.isFunction(value)) {
            let prevValue = _.get(data, name);
            if (prevValue === undefined) {
                let field = form.fields.get(name);
                if (field) {
                    prevValue = field.props.defaultValue;
                }
            }
            value = value(prevValue);
        }
        dispatch(actions.change(form.id, name, value));
    });

    const setFocus = (0, _reselect.defaultMemoize)((dispatch, form) => name => {
        let input = form.inputs.get(name);
        if (input) {
            focusInput(input);
        } else {
            setTimeout(() => {
                let input = form.inputs.get(name);
                if (input) {
                    focusInput(input);
                } else {
                    console.warn(`Field '${form.id}.${name}' was not found for focusing; did you set <input ref={this.props.focusRef}>?`);
                }
            }, 0);
        }
    });
    const data = Object.assign({}, ...state => _.get(state, 'data', _consts.emptyObject));
    const saveState = (0, _reselect.defaultMemoize)((dispatch, form, data) => () => {
        dispatch(actions.saveState(form.id, data));
    });

    //Exposed Form Connect Methods and Props
    return (state, props) => {
        let data = dataSelector(state, props);
        /*FIXME: Everything is working as expected and re-renders are happening when they should...
        but setField, saveState and isDirty cause a warning because a different closure is being returned
        each time a change happens */
        let nextProps = _extends({
            data,
            validate: validate(dispatch, props.form),
            setField: setField(dispatch, props.form, data),
            setFocus: setFocus(dispatch, props.form),
            saveState: saveState(dispatch, props.form, data),
            isDirty: dirtyDirtySelector(state, props)
        }, props);

        if ((0, _shallowEqual2.default)(prevProps, nextProps)) {
            return prevProps;
        }

        return prevProps = nextProps;
    };
}