'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = createReducer;

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _set = require('lodash/fp/set');

var _set2 = _interopRequireDefault(_set);

var _actionTypes = require('./actionTypes');

var _actionTypes2 = _interopRequireDefault(_actionTypes);

var _namespace = require('./namespace');

var _namespace2 = _interopRequireDefault(_namespace);

var _toPath = require('lodash/toPath');

var _toPath2 = _interopRequireDefault(_toPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function createReducer(data) {
    let initialState = {};

    if (data) {
        for (let formId of Object.keys(data)) {
            initialState[formId] = {
                data: data[formId],
                initial: data[formId]
            };
        }
    }

    return (state = initialState, action) => {
        let { payload } = action;
        switch (action.type) {
            case _actionTypes2.default.CHANGE:
                state = (0, _set2.default)([payload.formId, 'data', ...(0, _toPath2.default)(payload.name)], payload.value, state);
                state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'wasChanged'], true, state);
                break;
            case _actionTypes2.default.FOCUS:
                {
                    state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'isFocused'], payload.isFocused, state);
                    if (payload.isFocused) {
                        state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'wasFocused'], true, state);
                    } else if (!payload.isFocused) {
                        state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'wasBlurred'], true, state);
                    }
                    break;
                }
            case _actionTypes2.default.HOVER:
                {
                    state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'isHovering'], payload.isHovering, state);
                    if (payload.isHovering) {
                        state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'mouseEntered'], true, state);
                    } else if (!payload.isHovering) {
                        state = (0, _set2.default)([payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'mouseLeft'], true, state);
                    }
                    break;
                }
            case _actionTypes2.default.SUBMIT:
                {
                    let submitCount = _.get(state, [payload.formId, 'submit'], 0);
                    state = (0, _set2.default)([payload.formId, 'submit'], submitCount + 1, state);
                    break;
                }
            case _actionTypes2.default.ASYNC_VALIDATION:
                {
                    let path = [payload.formId, 'ui', ...(0, _toPath2.default)(payload.name), 'pendingValidations'];
                    let pending = _.get(state, path, 0);
                    state = (0, _set2.default)(path, payload.complete ? pending - 1 : pending + 1, state);
                    break;
                }
            case _actionTypes2.default.SAVE_STATE:
                state = (0, _set2.default)([payload.formId, 'initial'], payload.data, state);
                delete state[payload.formId]['ui'];
                break;
        }
        return state;
    };
};

/*
const ui = {
    isDirty: true, // value is different from initial state
    wasChanged: true, // value was changed
    isEmpty: true, // value is same as default value
    isFocused: true, // input is focused
    wasFocused: true,  // input was focused
    wasBlurred: true, // input lost focus
    isValid: true, // input has no validation errors
    isHovering: true, // mouse is over input
    mouseEntered: true, // mouse was over input
    mouseLeft: true, // mouse entered input and then left
    formValidated: true, // form submit was attempted while this input existed
    wasValid: true, // input was at some point valid
};
*/