'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.fileExists = exports.fileAccess = exports.fileStat = exports.readDir = exports.readJson = exports.readText = exports.writeFile = exports.readFile = undefined;
exports.getFiles = getFiles;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _array = require('./array');

var _promise = require('./promise');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// export * as default from './fs';
const readFile = exports.readFile = (0, _promise.promisify)(_fs2.default.readFile);
const writeFile = exports.writeFile = (0, _promise.promisify)(_fs2.default.writeFile);
const readText = exports.readText = file => readFile(file, { encoding: 'utf8' });
const readJson = exports.readJson = file => readText(file).then(x => JSON.parse(x));
const _readDir = (0, _promise.promisify)(_fs2.default.readdir);
const readDir = exports.readDir = path => _readDir(path).then(entries => entries.map(e => _path2.default.join(path, e)));
const fileStat = exports.fileStat = (0, _promise.promisify)(_fs2.default.stat);
const _fileAccess = (0, _promise.promisify)(_fs2.default.access);
const fileAccess = exports.fileAccess = (path, mode) => _fileAccess(path, mode).then(() => true, err => {
    // if file does not exist or permission is denied, return false, otherwise throw
    if (err.code === 'ENOENT' || err.code === 'EACCES') {
        return false;
    }
    throw err;
});
const fileExists = exports.fileExists = file => fileAccess(file, _fs2.default.constants.F_OK);

/**
 *
 * @param {string} dir
 * @param {Boolean} recursive
 * @returns {Promise.<string[]>}
 */
async function getFiles(dir, recursive = true) {
    let paths = await readDir(dir);

    if (recursive) {
        return Promise.all(paths.map(path => fileStat(path).then(stat => stat.isDirectory() ? getFiles(path, recursive) : [path]))).then(_array.flatten);
    }

    return _array.filterAsync.call(paths, p => fileStat(p).then(s => s.isFile()));
}