'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.isNativeFunction = isNativeFunction;
exports.isFunction = isFunction;
exports.isString = isString;
exports.isNumber = isNumber;
exports.isPromise = isPromise;
exports.isBoolean = isBoolean;
exports.isRegExp = isRegExp;
exports.isDate = isDate;
exports.isSet = isSet;
exports.isMap = isMap;
exports.isWeakMap = isWeakMap;
exports.isArray = isArray;
exports.isNull = isNull;
exports.isUndefined = isUndefined;
exports.isObject = isObject;
exports.isPlainObject = isPlainObject;
exports.isSymbol = isSymbol;
exports.isBuffer = isBuffer;
exports.isError = isError;
function isNativeFunction(obj) {
    return isFunction(obj) && obj.toString().endsWith('{ [native code] }');
}

function isFunction(obj) {
    return typeof obj === 'function';
}

function isString(obj) {
    return typeof obj === 'string' || obj instanceof String;
}

function isNumber(obj) {
    return typeof obj === 'number' || obj instanceof Number;
}

function isPromise(obj) {
    return obj instanceof Promise;
}

function isBoolean(obj) {
    return obj === true || obj === false; // there's also a `Boolean` type but it doesn't behave much like a boolean
}

function isRegExp(obj) {
    return obj instanceof RegExp;
}

function isDate(obj) {
    return obj instanceof Date;
}

function isSet(obj) {
    return obj instanceof Set;
}

function isMap(obj) {
    return obj instanceof Map;
}

function isWeakMap(obj) {
    return obj instanceof WeakMap;
}

function isArray(obj) {
    return Array.isArray(obj);
}

function isNull(obj) {
    return obj === null;
}

function isUndefined(obj) {
    return obj === undefined;
}

function isObject(obj) {
    return obj !== null && typeof obj === 'object';
}

function isPlainObject(obj) {
    return isObject(obj) && (obj.constructor === Object // obj = {}
    || obj.constructor === undefined // obj = Object.create(null)
    );
}

function isSymbol(obj) {
    return Object.prototype.toString.call(obj) === '[object Symbol]';
}

function isBuffer(obj) {
    if (true) {
        return Buffer.isBuffer(obj);
    }
    // TODO: should we check if obj is a UInt8Array or something?
    return false;
}

function isError(obj) {
    return obj instanceof Error;
}