'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.promisifyAll = undefined;
exports.promisify = promisify;
exports.allSettled = allSettled;

var _isType = require('./isType');

var _bindable = require('./bindable');

var _bindable2 = _interopRequireDefault(_bindable);

var _function = require('./function');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Returns a function that will wrap the given `nodeFunction`. Instead of taking a callback, the returned function will return a promise whose fate is decided by the callback behavior of the given node function. The node function should conform to node.js convention of accepting a callback as last argument and calling that callback with error as the first argument and success value on the second argument.
 *
 * @param {Function} nodeFunction
 * @returns {Function}
 */
function promisify(nodeFunction) {
    return function (...args) {
        return new Promise((resolve, reject) => {
            nodeFunction(...args, function (err, data) {
                if (err) {
                    reject(err);
                } else {
                    resolve(data);
                }
            });
        });
    };
}

const promisifyAll = exports.promisifyAll = module => (0, _function.wrapMethods)(module, promisify);

const thenFinally = (0, _bindable2.default)((promise, callback) => {
    const res = () => promise;
    const fin = () => Promise.resolve(callback()).then(res);
    return promise.then(fin, fin);
});

// same API as Q: https://github.com/kriskowal/q/wiki/API-Reference#promiseallsettled
function allSettled(promises) {
    return Promise.all(promises.map(p => Promise.resolve(p).then(v => ({
        state: 'fulfilled',
        value: v
    }), r => ({
        state: 'rejected',
        reason: r
    }))));
}