'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.encodeParam = encodeParam;
exports.queryParams = queryParams;

var _fmap = require('./fmap');

var _fmap2 = _interopRequireDefault(_fmap);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function encodeParam(x) {
    if (x === true) {
        return '1';
    }
    if (x === false) {
        return '0';
    }
    if (x === undefined || x === null) {
        return '';
    }
    return encodeURIComponent(x);
}

function queryParams(params) {
    var _context;

    return (_context = Object.keys(params), _fmap2.default).call(_context, k => {
        if (params[k] === undefined) {
            return _fmap.__skip__;
        }
        if (Array.isArray(params[k])) {
            return params[k].map(val => `${encodeParam(k)}[]=${encodeParam(val)}`).join('&');
        }
        return `${encodeParam(k)}=${encodeParam(params[k])}`;
    }).join('&');
}

/**
 * Joins URLs together with /. Leaves leading and trailing slashes alone. Does not duplicate internal slashes.
 * 
 * @param {string[]} urls
 * @returns {string}
 */
function joinUrlPaths(...urls) {
    return urls.map((u, i) => {
        if (i > 0) {
            u = u.replace(/^\/+/g, '');
        }
        if (i < urls.length - 1) {
            u = u.replace(/\/+$/g, '');
        }
        return u;
    }).join('/');
}