function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import { isFunction, isString } from './isType';
import bindable from './bindable';
import { identity } from './value';
import { toArray } from './collection';

export var flatten = bindable(function (arrayOfArrays) {
    var _Array$prototype;

    return (_Array$prototype = Array.prototype).concat.apply(_Array$prototype, _toConsumableArray(arrayOfArrays));
});

export var map = bindable(function (array, callback) {
    return toArray(array).map(callback);
});

export var filterAsync = bindable(function (array, mapCb) {
    var filterCb = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : identity;

    return Promise.all(array.map(mapCb)).then(function (results) {
        return array.filter(function (_, i) {
            return filterCb(results[i]);
        });
    });
});

/**
 * Map and flatten
 */
export var flatMap = bindable(function (array, callback) {
    var _context;

    return (_context = map.call(array, callback), flatten).call(_context);
});

export var groupBy = bindable(function groupBy(array, grouper) {
    return array.reduce(function (acc, val, idx) {
        var key = grouper(val, idx);
        if (acc[key]) {
            acc[key].push(val);
        } else {
            acc[key] = [val];
        }
        return acc;
    }, Object.create(null));
});

export var thru = bindable(function (array, callback) {
    return callback(array);
});

export var tap = bindable(function (array, callback) {
    callback(array);
    return array;
});

/**
 * Removes one instance of `value` from `array`, without mutating the original array. Uses loose comparison.
 *
 * @param {Array} array Array to remove value from
 * @param {*|Function} value Value to remove
 * @returns {Array} Array with `value` removed
 */
export function arrayWithout(array, value) {
    var func = value;
    if (typeof value !== 'function') {
        func = function func(v, i) {
            return v == value;
        };
    }
    for (var i = 0; i < array.length; ++i) {
        if (func(array[i], i)) {
            return arraySplice(array, i);
        }
    }
    return array;
}

/**
 * Removes an index from an array without mutating the original array.
 *
 * @param {Array} array Array to remove value from
 * @param {Number} index Index to remove
 * @param {Number} count
 * @param {Array} replaceWith
 * @returns {Array} Array with `value` removed
 */
export function arraySplice(array, index) {
    var count = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;
    var replaceWith = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];

    if (index < array.length) {
        var copy = [].concat(_toConsumableArray(array));
        copy.splice.apply(copy, [index, count].concat(_toConsumableArray(replaceWith)));
        return copy;
    }
    return array;
}

/**
 * Checks if `value` is in collection. Uses loose comparison.
 *
 * @param {Array} array
 * @param {*} value
 * @returns {boolean}
 */
export function arrayIncludes(array, value) {
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = array[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var val = _step.value;

            if (val == value) {
                return true;
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return false;
}