var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import React from 'react';
import PropTypes from 'prop-types';
import { connect, connectAdvanced } from 'react-redux';
import * as util from './util';
import * as _ from 'lodash';
var toPath = _.toPath;

import getOr from 'lodash/fp/getOr';
import { compose, mapProps, getContext, toClass, withProps, withPropsOnChange, pure, shouldUpdate } from 'recompose';
import namespace from './namespace';
import { createSelector, defaultMemoize, createSelectorCreator } from 'reselect';
import shallowEqual from './shallowEqual';

import * as actions from './actionCreators';
import DeepMap from './DeepMap';
import { emptyObject, emptyArray } from './consts';

var defaultValueGetter = function defaultValueGetter(_, p) {
    return p.defaultValue;
};
var stateGetter = function stateGetter(s, p) {
    return _.get(s, [namespace, p.form.id], emptyObject);
};

var ruleCache = new DeepMap();
// const notFound = Symbol('NotFound');
// const noError = Symbol('NoError');
// const pending = Symbol('Pending');

var errTypeToProp = {
    // 'err': 'errors',
    'error': 'errors',
    // 'warn': 'warnings',
    'warning': 'warnings'
};

// window.RULE_CACHE = ruleCache;

function getErrorMessages(result, rule, args) {
    if (result === true) {
        return emptyArray;
    }
    if (result === false) {
        if (_.isFunction(rule.message)) {
            return util.array(rule.message.apply(rule, _toConsumableArray(args)));
        } else {
            return util.array(rule.message);
        }
    }
    if (result) {
        // if result is not a boolean, it ought to be an error message or blank
        return util.array(result);
    }
    return emptyArray;
}

function getErrors(value, rules, formData, dispatch, formId, name, pendingValidations) {
    var getValue = function getValue(f) {
        return _.get(formData, f);
    };

    // console.log(formId,name,pendingValidations);

    var props = {
        errors: [],
        warnings: []
    };

    if (rules.length > 1) {
        // sort async functions to the end so that they can be skipped if other validation rules are already failing
        rules = [].concat(_toConsumableArray(rules)).sort(function (a, b) {
            if (a.isAsync === b.isAsync) return 0;
            return a.isAsync ? 1 : -1;
        });
    }

    var _loop = function _loop(rule) {
        if (rule.isOptional && util.isEmpty(value)) {
            return 'continue';
        }

        var dependsOn = util.array(rule.dependsOn);
        var args = [value];
        if (dependsOn.length) {
            args.push.apply(args, _toConsumableArray(dependsOn.map(getValue)));
        }

        if (!rule.precondition.apply(rule, args)) {
            return 'continue';
        }

        var errKey = errTypeToProp[rule.type];

        if (!errKey) {
            throw new Error('Unsupported rule type: ' + rule.type);
        }
        var cacheKey = [rule, formId, name];

        if (rule.isAsync) {
            cacheKey.push.apply(cacheKey, args);
        }

        var result = ruleCache.get(cacheKey);

        if (result) {
            if (rule.isAsync) {
                var _props$errKey;

                // fixme: rule.compare isn't used here because we want to cache more than one entry
                (_props$errKey = props[errKey]).push.apply(_props$errKey, _toConsumableArray(result));
                return 'continue';
            } else {
                var _result = _slicedToArray(result, 2),
                    lastArgs = _result[0],
                    lastMessages = _result[1];

                if (rule.compare(args, lastArgs)) {
                    var _props$errKey2;

                    (_props$errKey2 = props[errKey]).push.apply(_props$errKey2, _toConsumableArray(lastMessages));
                    return 'continue';
                }
            }
        }

        if (rule.isAsync) {
            if (rule.type === 'error' && props[errKey].length) {
                // skip if other error rules are already failing
                return 'continue';
            }

            ruleCache.set(cacheKey, emptyArray);
            dispatch(actions.asyncValidation(formId, name, false));
            rule.validate.apply(rule, args).then(function (result) {
                ruleCache.set(cacheKey, getErrorMessages(result, rule, args));
                dispatch(actions.asyncValidation(formId, name, true));
            }, function () {
                ruleCache.delete(cacheKey);
                dispatch(actions.asyncValidation(formId, name, true));
            });
        } else {
            var _props$errKey3;

            var _result2 = rule.validate.apply(rule, args);
            var messages = getErrorMessages(_result2, rule, args);
            (_props$errKey3 = props[errKey]).push.apply(_props$errKey3, _toConsumableArray(getErrorMessages(_result2, rule, args)));
            ruleCache.set(cacheKey, [args, messages]);
        }
    };

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = rules[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var rule = _step.value;

            var _ret = _loop(rule);

            if (_ret === 'continue') continue;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return props;
}

function mapDispatchToProps(dispatch, formId, name) {
    return {
        actions: {
            change: function change(value) {
                dispatch(actions.change(formId, name, value));
            }
        },
        events: {
            onChange: function onChange(ev) {
                dispatch(actions.change(formId, name, ev.target.value));
            },
            onCheck: function onCheck(ev) {
                dispatch(actions.change(formId, name, ev.target.checked));
            },

            // TODO: add onCheckList for checkbox arrays
            onFocus: function onFocus() {
                dispatch(actions.focus(formId, name));
            },
            onBlur: function onBlur() {
                dispatch(actions.blur(formId, name));
            },
            onMouseEnter: function onMouseEnter() {
                dispatch(actions.mouseEnter(formId, name));
            },
            onMouseLeave: function onMouseLeave() {
                dispatch(actions.mouseLeave(formId, name));
            },
            onSubmit: function onSubmit() {
                dispatch(actions.submit(formId));
            }
        }
    };
}

export default function connectField() {
    return compose(toClass, getContext({ form: PropTypes.object }), withPropsOnChange(['name', 'rules', 'formId'], function (props) {
        // console.log(props.name,'changing');
        var form = props.form || {
            id: props.formId,
            rules: [],
            fields: null,
            inputs: null
        };
        var fieldRules = util.array(props.rules);
        var baseRules = form.rules ? util.array(util.glob(form.rules, props.name, [])) : [];
        return {
            form: form,
            rules: _.concat(baseRules, fieldRules)
        };
    }),
    // connect(require('./mapStateToProps'), require('./mapDispatchToProps')),
    connectAdvanced(selectorFactory), withPropsOnChange(['name'], function (_ref) {
        var name = _ref.name,
            form = _ref.form;

        var props = {};
        if (form.fields) {
            props.ref = function (node) {
                if (node) {
                    form.fields.set(name, node);
                } else {
                    form.fields.delete(name);
                }
            };
        }
        if (form.inputs) {
            props.focusRef = function (node) {
                if (node) {
                    form.inputs.set(name, node);
                } else {
                    form.inputs.delete(name);
                }
            };
        }
        return props;
    }));
}

function selectorFactory(dispatch, factoryOptions) {
    var dispatchSelector = defaultMemoize(mapDispatchToProps);
    var prevProps = emptyObject;

    var namePathSelector = createSelector(function (_, p) {
        return p.name;
    }, toPath);
    var stateUiSelector = util.createDeepEqualSelector([stateGetter, namePathSelector], function (state, np) {
        return Object.assign({
            isFocused: false,
            isHovering: false,
            mouseEntered: false,
            mouseLeft: false,
            wasFocused: false,
            wasBlurred: false,
            wasChanged: false,
            pendingValidations: 0
        }, _.get(state, ['ui'].concat(_toConsumableArray(np)), emptyObject));
    });
    var dataGetter = createSelector(stateGetter, getOr(emptyObject, 'data'));
    var valueSelector = createSelector([dataGetter, namePathSelector, defaultValueGetter], function (data, np, dv) {
        return _.get(data, np, dv);
    });
    var initialSelector = createSelector(stateGetter, getOr(emptyObject, 'initial'));
    var initialValueSelector = createSelector([initialSelector, namePathSelector, defaultValueGetter], function (init, np, dv) {
        return _.get(init, np, dv);
    });
    var pendingValidationsSelector = createSelector(stateUiSelector, function (ui) {
        return ui.pendingValidations;
    });

    var errorSelector = util.createDeepEqualSelector([valueSelector, function (_, p) {
        return p.rules;
    }, dataGetter, function () {
        return dispatch;
    }, function (_, p) {
        return p.form.id;
    }, function (_, p) {
        return p.name;
    }, pendingValidationsSelector], getErrors);

    var isDirtySelector = createSelector([valueSelector, initialValueSelector], function (value, initialValue) {
        return !_.isEqual(value, initialValue);
    });
    // const isValidSelector = createSelector(errorSelector, errors => errors.length === 0);
    var isEmptySelector = createSelector([valueSelector, defaultValueGetter], _.isEqual);
    var formValidatedSelector = createSelector(stateGetter, function (state) {
        return _.get(state, 'submit', 0) > 0;
    });

    var uiSelector = util.createDeepEqualSelector([stateUiSelector, isDirtySelector, isEmptySelector, formValidatedSelector], function (ui, isDirty, isEmpty, formValidated) {
        return {
            isFocused: ui.isFocused,
            isHovering: ui.isHovering,
            mouseEntered: ui.mouseEntered,
            mouseLeft: ui.mouseLeft,
            wasFocused: ui.wasFocused,
            wasBlurred: ui.wasBlurred,
            wasChanged: ui.wasChanged,
            isValidating: ui.pendingValidations > 0,
            isDirty: isDirty,
            isEmpty: isEmpty,
            formValidated: formValidated
        };
    });

    return function (state, props) {
        var nextProps = _extends({
            value: valueSelector(state, props),
            ui: uiSelector(state, props)
        }, errorSelector(state, props), dispatchSelector(dispatch, props.form.id, props.name), props);

        if (shallowEqual(prevProps, nextProps)) {
            return prevProps;
        }

        return prevProps = nextProps;
    };
}