var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import React from 'react';
import PropTypes from 'prop-types';
import { connect, connectAdvanced } from 'react-redux';
import * as util from './util';
import * as _ from 'lodash';
import { compose, mapProps, getContext, withContext, lifecycle } from 'recompose';
import namespace from './namespace';
import * as actions from './actionCreators';
import ShortId from 'shortid';
import { createSelector, defaultMemoize } from 'reselect';
import shallowEqual from './shallowEqual';
import { emptyObject, emptyArray } from './consts';

var stateGetter = function stateGetter(s, p) {
    return _.get(s, [namespace, p.form.id], emptyObject);
};

function focusInput(input) {
    input.focus(); // TODO: check if input is an element, otherwise throw warning if it's a React component
    if (input.scrollIntoViewIfNeeded) {
        input.scrollIntoViewIfNeeded(false);
    }
}

var contextTypes = {
    form: PropTypes.object
};

export default function connectForm(options) {
    return compose(withContext(contextTypes, function (props) {
        var rules = props.rules || options.rules;
        return {
            form: {
                id: props.id || options.id || ShortId.generate(),
                rules: rules ? util.unflatten(rules) : emptyObject,
                fields: new Map(),
                inputs: new Map()
            }
        };
    }), getContext(contextTypes),
    // connect(mapStateToProps, mapDispatchToProps),
    connectAdvanced(selectorFactory));
}

function selectorFactory(dispatch, factoryOptions) {
    var dataSelector = createSelector(stateGetter, function (state) {
        return _.get(state, 'data', emptyObject);
    });
    var initialSelector = createSelector(stateGetter, function (state) {
        return _.get(state, 'initial', emptyObject);
    });
    var dirtySelector = defaultMemoize(function (d, i) {
        return !_.isEqual(d, i);
    });
    var dirtyDirtySelector = createSelector(dataSelector, initialSelector, function (d, i) {
        return function () {
            return dirtySelector(d, i);
        };
    });
    var prevProps = {};

    //Memoized Methods

    var validate = defaultMemoize(function (dispatch, form) {
        return function () {
            dispatch(actions.submit(form.id));
            return Array.from(form.fields.values()).every(function (f) {
                return f.props.errors.length === 0;
            });
        };
    });

    var setField = defaultMemoize(function (dispatch, form) {
        return function (name, value, data) {
            if (_.isFunction(value)) {
                var prevValue = _.get(data, name);
                if (prevValue === undefined) {
                    var field = form.fields.get(name);
                    if (field) {
                        prevValue = field.props.defaultValue;
                    }
                }
                value = value(prevValue);
            }
            dispatch(actions.change(form.id, name, value));
        };
    });

    var setFocus = defaultMemoize(function (dispatch, form) {
        return function (name) {
            var input = form.inputs.get(name);
            if (input) {
                focusInput(input);
            } else {
                setTimeout(function () {
                    var input = form.inputs.get(name);
                    if (input) {
                        focusInput(input);
                    } else {
                        console.warn('Field \'' + form.id + '.' + name + '\' was not found for focusing; did you set <input ref={this.props.focusRef}>?');
                    }
                }, 0);
            }
        };
    });
    var data = Object.assign.apply(Object, [{}].concat(_toConsumableArray(function (state) {
        return _.get(state, 'data', emptyObject);
    })));
    var saveState = defaultMemoize(function (dispatch, form, data) {
        return function () {
            dispatch(actions.saveState(form.id, data));
        };
    });

    //Exposed Form Connect Methods and Props
    return function (state, props) {
        var data = dataSelector(state, props);
        /*FIXME: Everything is working as expected and re-renders are happening when they should...
        but setField, saveState and isDirty cause a warning because a different closure is being returned
        each time a change happens */
        var nextProps = _extends({
            data: data,
            validate: validate(dispatch, props.form),
            setField: setField(dispatch, props.form, data),
            setFocus: setFocus(dispatch, props.form),
            saveState: saveState(dispatch, props.form, data),
            isDirty: dirtyDirtySelector(state, props)
        }, props);

        if (shallowEqual(prevProps, nextProps)) {
            return prevProps;
        }

        return prevProps = nextProps;
    };
}