function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import * as _ from 'lodash';
import setIn from 'lodash/fp/set';
import actionTypes from './actionTypes';
import namespace from './namespace';
import toPath from 'lodash/toPath';

export default function createReducer(data) {
    var initialState = {};

    if (data) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
            for (var _iterator = Object.keys(data)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var formId = _step.value;

                initialState[formId] = {
                    data: data[formId],
                    initial: data[formId]
                };
            }
        } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
        } finally {
            try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                }
            } finally {
                if (_didIteratorError) {
                    throw _iteratorError;
                }
            }
        }
    }

    return function () {
        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
        var action = arguments[1];
        var payload = action.payload;

        switch (action.type) {
            case actionTypes.CHANGE:
                state = setIn([payload.formId, 'data'].concat(_toConsumableArray(toPath(payload.name))), payload.value, state);
                state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['wasChanged']), true, state);
                break;
            case actionTypes.FOCUS:
                {
                    state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['isFocused']), payload.isFocused, state);
                    if (payload.isFocused) {
                        state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['wasFocused']), true, state);
                    } else if (!payload.isFocused) {
                        state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['wasBlurred']), true, state);
                    }
                    break;
                }
            case actionTypes.HOVER:
                {
                    state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['isHovering']), payload.isHovering, state);
                    if (payload.isHovering) {
                        state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['mouseEntered']), true, state);
                    } else if (!payload.isHovering) {
                        state = setIn([payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['mouseLeft']), true, state);
                    }
                    break;
                }
            case actionTypes.SUBMIT:
                {
                    var submitCount = _.get(state, [payload.formId, 'submit'], 0);
                    state = setIn([payload.formId, 'submit'], submitCount + 1, state);
                    break;
                }
            case actionTypes.ASYNC_VALIDATION:
                {
                    var path = [payload.formId, 'ui'].concat(_toConsumableArray(toPath(payload.name)), ['pendingValidations']);
                    var pending = _.get(state, path, 0);
                    state = setIn(path, payload.complete ? pending - 1 : pending + 1, state);
                    break;
                }
            case actionTypes.SAVE_STATE:
                state = setIn([payload.formId, 'initial'], payload.data, state);
                delete state[payload.formId]['ui'];
                break;
        }
        return state;
    };
};

/*
const ui = {
    isDirty: true, // value is different from initial state
    wasChanged: true, // value was changed
    isEmpty: true, // value is same as default value
    isFocused: true, // input is focused
    wasFocused: true,  // input was focused
    wasBlurred: true, // input lost focus
    isValid: true, // input has no validation errors
    isHovering: true, // mouse is over input
    mouseEntered: true, // mouse was over input
    mouseLeft: true, // mouse entered input and then left
    formValidated: true, // form submit was attempted while this input existed
    wasValid: true, // input was at some point valid
};
*/