import { isFunction } from './isType';
import bindable from './bindable';
import { wrapMethods } from './function';

/**
 * Returns a function that will wrap the given `nodeFunction`. Instead of taking a callback, the returned function will return a promise whose fate is decided by the callback behavior of the given node function. The node function should conform to node.js convention of accepting a callback as last argument and calling that callback with error as the first argument and success value on the second argument.
 *
 * @param {Function} nodeFunction
 * @returns {Function}
 */
export function promisify(nodeFunction) {
    return function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
        }

        return new Promise(function (resolve, reject) {
            nodeFunction.apply(undefined, args.concat([function (err, data) {
                if (err) {
                    reject(err);
                } else {
                    resolve(data);
                }
            }]));
        });
    };
}

export var promisifyAll = function promisifyAll(module) {
    return wrapMethods(module, promisify);
};

var thenFinally = bindable(function (promise, callback) {
    var res = function res() {
        return promise;
    };
    var fin = function fin() {
        return Promise.resolve(callback()).then(res);
    };
    return promise.then(fin, fin);
});

// same API as Q: https://github.com/kriskowal/q/wiki/API-Reference#promiseallsettled
export function allSettled(promises) {
    return Promise.all(promises.map(function (p) {
        return Promise.resolve(p).then(function (v) {
            return {
                state: 'fulfilled',
                value: v
            };
        }, function (r) {
            return {
                state: 'rejected',
                reason: r
            };
        });
    }));
}