function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var _ = require('lodash');
var classNames = require('classnames');

var _require = require('reselect'),
    defaultMemoize = _require.defaultMemoize,
    createSelectorCreator = _require.createSelectorCreator;

/**
 * Unflatten an object. e.g.,
 *
 *      > unflatten({
 *          "foo.bar": "baz"
 *      })
 *      {
 *          foo: {
 *              bar: "baz"
 *          }
 *      }
 *
 * @param {Object} object Flat object
 * @returns {Object} Unflattened object
 */


export function unflatten(object) {
    var result = {};
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = Object.keys(object)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var key = _step.value;

            _.set(result, key, object[key]);
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return result;
};

var _glob = function _glob(object, path, defaultValue) {
    var _path = _toArray(path),
        first = _path[0],
        rest = _path.slice(1);

    var value = object[first];
    if (value === undefined) {
        value = object['*'];
        if (value === undefined) {
            // TODO: support **
            return defaultValue;
        }
    }
    if (rest.length) {
        return _glob(value, rest, defaultValue);
    }
    return value;
};

/**
 * Gets the value at `path` of `object`. If the resolved value is `undefined`, look for a `*` key. If that too is `undefined`, the `defaultValue` is returned in its place.
 *
 * @param {Object} object The object to query.
 * @param {Array|string} path The path of the property to get.
 * @param {*} defaultValue The value returned for `undefined` resolved values.
 * @return {*} Returns the resolved value.
 * @see https://lodash.com/docs/4.17.4#get
 */
export function glob(object, path, defaultValue) {
    if (!object) {
        console.warn('glob object is falsey');
        return defaultValue;
    }
    return _glob(object, _.toPath(path), defaultValue);
}

export function mergeAttrs(merged) {
    var eventHandlers = {};

    for (var _len = arguments.length, attrDicts = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        attrDicts[_key - 1] = arguments[_key];
    }

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = attrDicts[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var attrs = _step2.value;

            _.each(attrs, function (value, attr) {
                if (value === undefined) {
                    //
                } else if (merged[attr] === undefined) {
                    merged[attr] = value;
                } else if (attr === 'style') {
                    Object.assign(merged[attr], value);
                } else if (attr === 'className') {
                    merged[attr] = classNames(merged[attr], value);
                } else if (attr === 'ref' || /^on[A-Z]/.test(attr)) {
                    (eventHandlers[attr] || (eventHandlers[attr] = [])).push(value);
                } else {
                    merged[attr] = value;
                }
            });
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    _.each(eventHandlers, function (funcs, attr) {
        if (merged[attr]) {
            funcs.unshift(merged[attr]);
        }
        if (funcs.length === 1) {
            merged[attr] = funcs[0];
        } else {
            merged[attr] = function () {
                for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                    args[_key2] = arguments[_key2];
                }

                var result = undefined;
                var _iteratorNormalCompletion3 = true;
                var _didIteratorError3 = false;
                var _iteratorError3 = undefined;

                try {
                    for (var _iterator3 = funcs[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                        var func = _step3.value;

                        result = func.apply(undefined, args.concat([result]));
                    }
                } catch (err) {
                    _didIteratorError3 = true;
                    _iteratorError3 = err;
                } finally {
                    try {
                        if (!_iteratorNormalCompletion3 && _iterator3.return) {
                            _iterator3.return();
                        }
                    } finally {
                        if (_didIteratorError3) {
                            throw _iteratorError3;
                        }
                    }
                }

                return result;
            };
        }
    });

    return merged;
}

/**
 * Returns true if a value is null, undefined, or NaN.
 *
 * @param {*} value Value to check
 * @returns {boolean} True if a value is null, undefined, or NaN.
 * @see https://github.com/graphql/graphql-js/blob/39744381d5173795d3b245dcb5d86e78bb3638fe/src/jsutils/isNullish.js#L11-L16
 */
export function isNullish(value) {
    // eslint-disable-next-line no-self-compare
    return value === null || value === undefined || value !== value;
}

export function array(x) {
    if (!x) return [];
    return Array.isArray(x) ? [].concat(_toConsumableArray(x)) : [x];
}

export var createDeepEqualSelector = createSelectorCreator(defaultMemoize, _.isEqual);

export function pick(src, props) {
    var dest = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    return Object.keys(props).reduce(function (dest, p) {
        if (typeof props[p] === 'string') {
            dest[props[p]] = src[p];
        } else if (props[p]) {
            dest[p] = src[p];
        }
        return dest;
    }, dest);
}

export function arrayCompare(arr1, arr2) {
    return arr1.length === arr2.length && arr1.every(function (v, i) {
        return Object.is(v, arr2[i]);
    });
}

export function isEmpty(value) {
    if (_.isString(value) || _.isArray(value)) {
        return !value.length;
    }
    if (value instanceof Map || value instanceof Set) {
        return !value.size;
    }
    if (_.isPlainObject(value)) {
        return !_.size(value);
    }
    return !value;
}